@@ -99,7 +99,9 @@ public:
 		TEST_ADD(UriSuite::testCrash_MakeOwner_Bug20080207)
 		TEST_ADD(UriSuite::testQueryList)
 		TEST_ADD(UriSuite::testQueryListPair)
+		TEST_ADD(UriSuite::testQueryDissection_Bug3590761)
 		TEST_ADD(UriSuite::testFreeCrash_Bug20080827)
+		TEST_ADD(UriSuite::testParseInvalid_Bug16)
 	}
 
 private:
@@ -143,6 +145,8 @@ Rule                                | Example | hostSet | absPath | emptySeg
 1) URI = scheme ":" hier-part ...   |         |         |         |
    1) "//" authority path-abempty   | "s://"  | true    |   false |   false
                                     | "s:///" | true    |   false | true
+                                    | "s://a" | true    |   false | false
+                                    | "s://a/"| true    |   false | true
    2) path-absolute                 | "s:/"   |   false | true    |   false
    3) path-rootless                 | "s:a"   |   false |   false |   false
                                     | "s:a/"  |   false |   false | true
@@ -159,6 +163,8 @@ Rule                                | Example | hostSet | absPath | emptySeg
 		*/
 		TEST_ASSERT(testDistinctionHelper("s://", true, false, false));
 		TEST_ASSERT(testDistinctionHelper("s:///", true, false, true));
+		TEST_ASSERT(testDistinctionHelper("s://a", true, false, false));
+		TEST_ASSERT(testDistinctionHelper("s://a/", true, false, true));
 		TEST_ASSERT(testDistinctionHelper("s:/", false, true, false));
 		TEST_ASSERT(testDistinctionHelper("s:a", false, false, false));
 		TEST_ASSERT(testDistinctionHelper("s:a/", false, false, true));
@@ -1094,6 +1100,10 @@ Rule                                | Example | hostSet | absPath | emptySeg
 		TEST_ASSERT(testAddBaseHelper(L"http://a/b/c/d;p?q", L"g#s/./x", L"http://a/b/c/g#s/./x"));
 		TEST_ASSERT(testAddBaseHelper(L"http://a/b/c/d;p?q", L"g#s/../x", L"http://a/b/c/g#s/../x"));
 		TEST_ASSERT(testAddBaseHelper(L"http://a/b/c/d;p?q", L"http:g", L"http:g"));
+
+		// Bug related to absolutePath flag set despite presence of host
+		TEST_ASSERT(testAddBaseHelper(L"http://a/b/c/d;p?q", L"/", L"http://a/"));
+		TEST_ASSERT(testAddBaseHelper(L"http://a/b/c/d;p?q", L"/g/", L"http://a/g/"));
 	}
 
 	bool testToStringHelper(const wchar_t * text) {
@@ -1610,7 +1620,7 @@ Rule                                | Example | hostSet | absPath | emptySeg
 					&charsWritten, spacePlusConversion, normalizeBreaks);
 			TEST_ASSERT(res == URI_SUCCESS);
 			TEST_ASSERT(charsWritten <= charsRequired);
-			TEST_ASSERT(charsWritten = (int)wcslen(input) + 1);
+			TEST_ASSERT(charsWritten == (int)wcslen(input) + 1);
 			TEST_ASSERT(!wcscmp(input, recomposed));
 			delete [] recomposed;
 
@@ -1618,7 +1628,7 @@ Rule                                | Example | hostSet | absPath | emptySeg
 			res = uriComposeQueryMallocW(&recomposed, queryList);
 			TEST_ASSERT(res == URI_SUCCESS);
 			TEST_ASSERT(recomposed != NULL);
-			TEST_ASSERT(charsWritten = (int)wcslen(input) + 1);
+			TEST_ASSERT(charsWritten == (int)wcslen(input) + 1);
 			TEST_ASSERT(!wcscmp(input, recomposed));
 			free(recomposed);
 		}
@@ -1644,7 +1654,7 @@ Rule                                | Example | hostSet | absPath | emptySeg
 		res = uriDissectQueryMallocA(&queryList, &itemCount, pair, pair + strlen(pair));
 		TEST_ASSERT(res == URI_SUCCESS);
 		TEST_ASSERT(queryList != NULL);
-		TEST_ASSERT(itemCount = 1);
+		TEST_ASSERT(itemCount == 1);
 		TEST_ASSERT(!strcmp(queryList->key, unescapedKey));
 		TEST_ASSERT(!strcmp(queryList->value, unescapedValue));
 
@@ -1663,6 +1673,31 @@ Rule                                | Example | hostSet | absPath | emptySeg
 		testQueryListPairHelper("one=two=three=four", "one", "two=three=four", "one=two%3Dthree%3Dfour");
 	}
 
+	void testQueryDissection_Bug3590761() {
+		int res;
+		UriQueryListA * queryList;
+		int itemCount;
+		const char * const pair = "q=hello&x=&y=";
+
+		res = uriDissectQueryMallocA(&queryList, &itemCount, pair, pair + strlen(pair));
+		TEST_ASSERT(res == URI_SUCCESS);
+		TEST_ASSERT(queryList != NULL);
+		TEST_ASSERT(itemCount == 3);
+
+		TEST_ASSERT(!strcmp(queryList->key, "q"));
+		TEST_ASSERT(!strcmp(queryList->value, "hello"));
+
+		TEST_ASSERT(!strcmp(queryList->next->key, "x"));
+		TEST_ASSERT(!strcmp(queryList->next->value, ""));
+
+		TEST_ASSERT(!strcmp(queryList->next->next->key, "y"));
+		TEST_ASSERT(!strcmp(queryList->next->next->value, ""));
+
+		TEST_ASSERT(! queryList->next->next->next);
+
+		uriFreeQueryListA(queryList);
+	}
+
 	void testFreeCrash_Bug20080827() {
 		char const * const sourceUri = "abc";
 		char const * const baseUri = "http://www.example.org/";
@@ -1689,6 +1724,20 @@ Rule                                | Example | hostSet | absPath | emptySeg
 		uriFreeUriMembersA(&absoluteDest); // Crashed here
 	}
 
+	void testParseInvalid_Bug16() {
+		UriParserStateA stateA;
+		UriUriA uriA;
+		stateA.uri = &uriA;
+		const char * const input = "A>B";
+
+		const int res = uriParseUriA(&stateA, input);
+
+		TEST_ASSERT(res == URI_ERROR_SYNTAX);
+		TEST_ASSERT(stateA.errorPos == input + 1);
+		TEST_ASSERT(stateA.errorCode == URI_ERROR_SYNTAX);  /* failed previously */
+
+		uriFreeUriMembersA(&uriA);
+	}
 };
 
 
